@@ -16,7 +16,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Joerg.Schulenburg@physik.uni-magdeburg.de
+ see README for EMAIL-address
 
   sometimes I have written comments in german language, sorry for that
 
@@ -29,7 +29,12 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define __GOCR_H__
 
 #include "pnm.h"
+#include "unicode.h"
+#include "list.h"
 #include <stddef.h>
+#ifdef HAVE_GETTIMEOFDAY
+#include <sys/time.h>
+#endif
 
 /*
  *  wchar_t should always exist (ANSI), but WCHAR.H is sometimes missing
@@ -57,12 +62,15 @@ typedef enum direction DIRECTION;
 
 #define MAXlines 1024
 
+/* FIXME jb: remove tline, not used
 struct tline { 
    int x0,y0,x1,y1; 	// middle points
 };
+*/
 
 struct tlines {
     int num;
+    /* ToDo: add dx, so we have a vector (dx,dy) (js) */
     int dy;		/* offset for straight/skew lines dy(x+px) */
     int m1[MAXlines],   /* start of line = upper bound of 'A' */
         m2[MAXlines],   /* upper bound of 'e' */
@@ -72,31 +80,29 @@ struct tlines {
         x1[MAXlines];		/* left and right border */
 };
 
-
+#define NumAlt 10 // maximal number of alternative chars (table length)
 struct box {	// this structure should contain all pixel infos of a letter
     int x0,x1,y0,y1,x,y,dots;	// xmin,xmax,ymin,ymax,reference-pixel,i-dots
     wchar_t c,ac;	// detected (alternate) char
-    char modifier;	// default=0, see compose() in unicode.c
+    wchar_t modifier;	// default=0, see compose() in unicode.c
     int num;		// same nummer = same char
     int line;		// line number (points to struct tlines lines)
     int m1,m2,m3,m4;	// m2 = upper boundary, m3 = baseline
     // planed: sizeof hole_1, hole_2, certainty (run1=100%,run2=90%,etc.)
     pix *p;		// pointer to pixmap (v0.2.5)
+    /* tac, wac is used together with setac() to manage very similar chars */
+    int num_ac;         // length of table (alternative chars), default=0
+    wchar_t tac[NumAlt]; // alternative chars, only used by setac(),getac()
+    int     wac[NumAlt]; // weight of alternative chars
+    char    *obj;        // pointer to text-object ... (default=NULL)
+                         //  ... (melted chars, barcode, picture coords, ...)
+                         //  must be freed before box is freed!
+                         //  do _not_ copy obj !!!!!
 };
 typedef struct box Box;
 
 #define outbounds(p, tx, ty)   	    (tx < 0 || ty < 0 || tx >= (p)->x || ty >= (p)->y)
 
-struct environment {		// public for all
-    int cs;		// critical value (pixel<cs => black pixel)
-    int avX,avY;	// average X,Y (avX=sumX/numC)
-    int sumX,sumY,numC; // sum of all X,Y; nim chars
-    pix *p;		// pixelmap 8bit gray
-    int vvv;		// verbose option (see --help)
-    int mode;           // operation mode (see --help)
-    char *db_path;      // pathname for database
-};
-
 typedef struct path {
   int start;			// color at the beginning of the path
 				// (0=white, 1=black)
@@ -108,7 +114,62 @@ typedef struct path {
 				// the arrays are enlarged)
 } path_t;
 
-extern struct environment env;	/* declaration for other c-files */
+/* job_t contains all information needed for an OCR task */
+typedef struct job_s {
+  struct { /* source data */
+    char *fname; /* input filename; default value: "-" */
+    pix p; /* source pixel data */
+           /* pixelmap 8bit gray */
+  } src;
+  struct { /* temporary stuff, e.g. buffers */
+#ifdef HAVE_GETTIMEOFDAY
+    struct timeval init_time; /* starting time of this job */
+#endif
+    pix ppo; /* pixmap for visual debugging output */
+
+    /* sometimes recognition function is called again and again, if result was 0
+       n_run tells the pixel function to return alternative results */
+    int n_run; /* num of run, if run_2 critical pattern get other results */
+               /* used for 2nd try, pixel uses slower filter function etc. */
+    List dblist; /* FIXME jb: desc */
+  } tmp;
+  struct { /* results */
+    List boxlist; /* FIXME jb: desc */
+    List linelist; /* FIXME jb: desc */
+    
+    struct tlines lines; /* used to access to line-data */
+                         /* here the positions (frames) of lines are */
+                         /* stored for further use */
+
+    int avX,avY; /* average X,Y (avX=sumX/numC) */
+    int sumX,sumY,numC; /* sum of all X,Y; num chars */
+  } res;
+  struct { /* configuration */
+    int cs; /* FIXME jb: grey level 0<160<=255; default value: 0 */
+            /* critical value (pixel<cs => black pixel) */
+
+    int spc; /* spacewidth/dots (0 = autodetect); default value: 0 */
+    int mode; /* operation modes; default value: 0 */
+            /* operation mode (see --help) */
+    int dust_size; /* dust size; default value: 10 */
+    int only_numbers; /* numbers only; default value: 0 */
+    int verbose; /* verbose mode; default value: 0 */ 
+                 /* verbose option (see --help) */
+    FORMAT out_format; /* output format; default value: ISO8859_1*/
+    char *lc; /* list of chars (_ = not recognized chars); default value: "_" */
+    char *db_path; /* pathname for database; default value: NULL */
+    char *cfilter; /* char filter; default value: NULL */
+  } cfg;
+} job_t;
+
+/* initialze job structure */
+void job_init(job_t *job);
+
+/* free job structure */
+void job_free(job_t *job);
+
+//FIXME jb: remove JOB;
+extern job_t *JOB;
 
 // kalkulate the overlapp of the line (0-1) with black points 
 // by rekursiv bisection 
@@ -162,9 +223,17 @@ int joined(pix *p, int x0, int y0, int x1, int y1, int cs);
 // return number of steps
 int loop(pix *p, int x, int y, int l, int cs, int col, DIRECTION r);
 
+#define MAX_HOLES 3
+typedef struct list_holes { 
+  int num;    /* numbers of holes, initialize with 0 */ 
+  struct hole_s {
+    int size,x,y,x0,y0,x1,y1; /*  size, start point, outer rectangle */
+  } hole[MAX_HOLES];
+} holes_t;
+          
 // look for white holes surrounded by black points
 // at moment white point with black in all four directions
-int num_hole(int x0, int x1, int y0, int y1, pix *p, int cs);
+int num_hole(int x0, int x1, int y0, int y1, pix *p, int cs, holes_t *holes);
 
 // count for black nonconnected objects --- used for i,auml,ouml,etc.
 int num_obj(int x0, int x1, int y0, int y1, pix  *p, int cs);
