@@ -16,7 +16,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Joerg.Schulenburg@physik.uni-magdeburg.de */
+ see README for EMAIL-address */
  
 #ifndef OUTPUT_H
 #define OUTPUT_H
@@ -27,13 +27,9 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "gocr.h"
 #include "list.h"
 
-extern int n_run;
-extern List boxlist;
-
-void out_b(pix *b, int x0, int y0, int dx, int dy, int cs );
+void out_b(struct box *px, pix *b, int x0, int y0, int dx, int dy, int cs );
 void out_x(struct box *px);
 void out_x2(struct box *box1,struct box *box2);
-void out_b2(pix *b, int x0, int y0, int dx, int dy, int cs, int x1, int y1 );
 int output_list(pix * pp, char *lc);
 int write_img(char *fname, pix * ppo, pix * p, int opt);
 
