@@ -16,7 +16,9 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Joerg.Schulenburg@physik.uni-magdeburg.de */
+ see README for EMAIL-address
+
+*/
  
 #ifndef PGM2ASC_H
 #define PGM2ASC_H 1
@@ -29,6 +31,23 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #define pixel_at(pic, xx, yy)           (pic).p[(xx)+((yy)*((pic).x))]
 #define pixel_atp(pic, xx, yy)          (pic)->p[(xx)+((yy)*((pic)->x))]
 
+#ifndef HAVE_WCHAR_H
+wchar_t *wcschr (wchar_t *wcs, wchar_t wc);
+wchar_t *wcscpy (wchar_t *dest, const wchar_t *src);
+size_t wcslen (const wchar_t *s);
+#endif
+#ifndef HAVE_WCSDUP
+wchar_t * wcsdup (const wchar_t *WS);	/* its a gnu extension */
+#endif
+
+/* declared in pgm2asc.c */
+/* set alternate chars and its weight, called from the engine
+    if a char is recognized to (weight) percent */
+int setac(struct box *b, wchar_t ac, int weight);
+
+/* for qsort() call */
+int intcompare (const void *vr, const void *vs);
+
 /* declared in box.c */
 int box_gt(struct box *box1, struct box *box2);
 struct box *malloc_box( struct box *inibox );
@@ -41,9 +60,9 @@ wchar_t ocr_db(struct box *box1);
 /* declared in detect.c */
 int detect_lines1(pix * p, int x0, int y0, int dx, int dy);
 int detect_lines2(pix *p,int x0,int y0,int dx,int dy,int r);
-int detect_rotation_angle(pix *pp);
+int detect_rotation_angle(job_t *job);
 int detect_text_lines(pix * pp, int mo);
-int detect_pictures(void);
+int detect_pictures(job_t *job);
 
 /* declared in lines.c */
 void store_boxtree_lines( int mo );
@@ -58,29 +77,24 @@ void free_textlines( void );
     * If the line is out of range, the function returns 0,
     * otherwise a pointer to a complete line.
     */
-#ifdef USE_UNICODE
-const wchar_t *getTextLine( int );
-#else
 const char *getTextLine( int );
-#endif
 
 /* declared in remove.c */
-int remove_dust( pix *pp, int dust_size );
-int remove_pictures( );
+int remove_dust( job_t *job );
+int remove_pictures( job_t *job);
 int remove_melted_serifs( pix *pp );
 int remove_rest_of_dust();
+int smooth_borders( job_t *job );
 
 /* declared in pixel.c */
 int marked(pix * p, int x, int y);
 int pixel(pix *p, int x, int y);
 void put(pix * p, int x, int y, int ia, int io);
 
-   /* start ocr on a image in pix */
-   int pgm2asc(
-	       pix *pp,			// see pnm.h, pixelbuffer and size
-	       int mo, int cs, int spc,	// mode=0 cs=0 spacewidth=0
-	       int dust_size, char *lc,	// dust_size=10 listchars="_"
-	       int vvv			// verbose=0
-	       );
+/* start ocr on a image in job.src.p */
+int pgm2asc(job_t *job);
+
+/* initialization of progress output, fname="<fileID>","<filename>","-"  */
+int ini_progress(char *fname);
 
 #endif
