@@ -35,7 +35,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
- Joerg.Schulenburg@physik.uni-magdeburg.de
+ see README for EMAIL-address
  */
  
 /*
@@ -53,7 +53,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include <stddef.h>
 
 enum format {
-	ISO8859_1, TeX, HTML, SGML 
+	ISO8859_1, TeX, HTML, XML, SGML, UTF8, ASCII
 };
 typedef enum format FORMAT;
 
@@ -61,7 +61,7 @@ typedef enum format FORMAT;
  * Prototypes
  */
 wchar_t compose(wchar_t main, wchar_t modifier);
-const unsigned char *decode(wchar_t c, FORMAT type);
+const char *decode(wchar_t c, FORMAT type);
 
 /*
  * Unicode codes
@@ -978,6 +978,7 @@ const unsigned char *decode(wchar_t c, FORMAT type);
 #define PER_MILLE_SIGN							0x2030
 #define SINGLE_LEFT_POINTING_ANGLE_QUOTATION_MARK			0x2039
 #define SINGLE_RIGHT_POINTING_ANGLE_QUOTATION_MARK			0x203A
+#define EURO_CURRENCY_SIGN                                              0x20AC
 
 /* mathematical operators */
 #define FOR_ALL								0x2200
